@@ -11,7 +11,7 @@
 */
 
 /*
- * $Id: hash.c 8 2008-05-28 09:27:26Z rmanfredi $
+ * $Id: hash.c 58 2008-08-19 08:44:07Z rmanfredi $
  *
  *  Copyright (c) 1990-2006, Raphael Manfredi
  *  
@@ -30,9 +30,15 @@
 #include "config.h"
 #include "portable.h"
 
+#ifdef I_STDLIB
+#include <stdlib.h>
+#else
 #ifdef I_MALLOC
 #include <malloc.h>
+#else
+extern char *malloc();				/* Memory allocation */
 #endif
+#endif	/* I_STDLIB */
 
 #ifdef I_STRING
 #include <string.h>
@@ -46,7 +52,7 @@
 
 #ifndef lint
 private char *rcsid =
-	"$Id: hash.c 8 2008-05-28 09:27:26Z rmanfredi $";
+	"$Id: hash.c 58 2008-08-19 08:44:07Z rmanfredi $";
 #endif
 
 private uint32 hashcode();			/* The hahsing function */
