@@ -11,7 +11,7 @@
 */
 
 /*
- * $Id: logfile.c 8 2008-05-28 09:27:26Z rmanfredi $
+ * $Id: logfile.c 58 2008-08-19 08:44:07Z rmanfredi $
  *
  *  Copyright (c) 1990-2006, Raphael Manfredi
  *  
@@ -48,9 +48,15 @@
 #include <errno.h>
 #include <sys/types.h>
 
+#ifdef I_STDLIB
+#include <stdlib.h>
+#else
 #ifdef I_MALLOC
 #include <malloc.h>
+#else
+extern char *malloc();				/* Memory allocation */
 #endif
+#endif	/* I_STDLIB */
 
 #ifdef I_TIME
 # include <time.h>
