@@ -1,4 +1,4 @@
-;# $Id: biff.pl 1 2006-08-24 13:24:12Z rmanfredi $
+;# $Id: biff.pl 56 2008-07-01 07:33:24Z rmanfredi $
 ;#
 ;#  Copyright (c) 1990-2006, Raphael Manfredi
 ;#  
@@ -35,7 +35,7 @@
 ;# mailagent-specific daemon on another host is not easy to set-up, and that
 ;# daemon is vaporware today anyway].
 ;#
-;# This package relies on pl/utmp/utmp.pl.
+;# This package relies on pl/utmp/utmp.pl and pl/termios/termios.pl.
 #
 # Local biff support
 #
@@ -65,14 +65,29 @@ sub notify {
 	$tty = "/dev/$tty" unless $'test_mode;	# Re-anchor name in file system
 	return unless -x $tty;		# Return if no biffing wanted on that tty
 
-	&'add_log("biffing $cf'user on $tty") if $'loglvl > 8;
+	my ($row, $col) = termios'size($tty);
+	&'add_log("WARNING cannot compute size of $tty: $row")
+		if defined($row) && !defined($col) && $'loglvl > 3;
+	my $assuming = "";
+	unless (defined $col) {
+		($row, $col) = (24, 80);
+		$assuming = "assuming ";
+	}
+	&'add_log("biffing $cf'user on $tty ($assuming$row x $col)")
+		if $'loglvl > 8;
 
 	local($folder) = &'tilda($path);	# Replace home directory with a ~
 	local($n) = "\n\r";					# Use \r in case tty is in raw mode
 
+	# Biffing context containing the amount of lines we can still emit before
+	# reaching the size of the window, and the amount of columns we have for
+	# displaying the text.
+	local @context = ($row, $col);
+
 	unless (open(TTY, ">$tty")) {
-		&'add_log("ERROR cannot open $tty: $!") if $'loglvl;
-		&'add_log("WARNING unable to biff for $folder ($type)") if $'loglvl > 5;
+		&'add_log("ERROR cannot write on $tty: $!") if $'loglvl;
+		&'add_log("WARNING unable to biff for $folder ($type) on $tty")
+			if $'loglvl > 5;
 		return;
 	}
 
@@ -107,7 +122,7 @@ sub custom {
 	local($format, $type) = @_;
 	unless (open(FORMAT, $format)) {
 		&'add_log("ERROR cannot open biff format $format: $!") if $'loglvl > 1;
-		&default;		# Use default format then
+		&default;			# Use default format then
 		return;
 	}
 
@@ -174,9 +189,23 @@ P	$biff'fpath
 :	\02!
 EOM
 	local($_);
+	my $reformat = $cf'biffnice =~ /^on/i;
+	my $width = $context[1];
 	while (<FORMAT>) {
 		chop;
-		print TTY &'macros_subst(*_), $n;
+		my @lines = split($n, &'macros_subst(*_));
+		if (@lines) {
+			foreach my $l (@lines) {
+				if ($reformat) {
+					local @tmp;
+					&format($l, $width, *tmp);	# Format line into @tmp
+					$l = join($n, @tmp);
+				}
+				print TTY $l, $n;
+			}
+		} else {
+			print TTY $n;
+		}
 	}
 	close FORMAT;
 	&macro'unload;			# Release customized macros
@@ -188,8 +217,13 @@ sub beep { "\07" x $env'beep; }
 
 # Default biffing
 sub default {
-	print TTY "$n\07New $mtype for $cf'user has arrived in $folder:$n";
+	my $header = "New $mtype for $cf'user has arrived in $folder:";
+	my $width = $context[1];
+	my $lines = int(length($header) / $width) + 1;
+	$lines-- if 0 == length($header) % $width;
+	print TTY "$n\07$header$n";
 	print TTY "----$n";
+	$context[0] -= $lines + 1;		# Header line plus dashes
 	print TTY &all;
 	print TTY "$n----\07$n";
 }
@@ -207,15 +241,17 @@ sub all {
 }
 
 # Returns mail headers defined in @head, on the opened TTY
-# If the header length is greater than 79 characters, it is trimmed at 76 and
+# If the header length is greater than the tty width, it is trimmed and
 # three dots '...' are emitted to show something was truncated.
 # Also known as the %-H macro
 sub headers {
 	local($res) = '';
+	my $width = $context[1];		# tty columns
 	foreach $head (@head) {
 		next unless defined $'Header{$head};
-		local($line) = "$head: $'Header{$head}";
-		$line = substr($line, 0, 76) . '...' if length($line) >= 80;
+		local($line) = unquote_printable("$head: $'Header{$head}");
+		$line =~ s/[\x0-\x1f\x7f]//g;
+		$line = substr($line, 0, $width - 4) . '...' if length($line) >= $width;
 		$res .= "$line$n";
 	}
 	chop($res);			# Remove final \n\r for macro substitution
@@ -223,6 +259,12 @@ sub headers {
 	$res;
 }
 
+# Is line a blank one?
+sub is_blank {
+	my ($l) = @_;
+	return $l =~ /^[\W_]*$/;	# Contains only non-words and underscores
+}
+
 # Print first $cf'bifflines lines or $cf'bifflen charaters, whichever
 # comes first. Assumes TTY already opened correctly
 # Also known as the %-B macro if called body(0), or %-T if called body(1).
@@ -230,7 +272,7 @@ sub body {
 	local($trim) = @_;			# Whether top reply text should be trimmed
 	local($len) = defined $cf'bifflen ? $cf'bifflen : 560;
 	local($lines) = defined $cf'bifflines ? $cf'bifflines : 7;
-	local(@body) = split(/\n/, $'Header{'Body'});
+	local(@body) = split(/\r?\n/, ${$'Header{'=Body='}});
 	local($skipnl) = $cf'biffnl =~ /OFF/i;	# Skip blank lines?
 	local($_);
 	local($res) = '';
@@ -238,11 +280,39 @@ sub body {
 	# Setting bifflen or bifflines to 0 means no body
 	return '' if $len == 0 || $lines == 0;
 
+	my ($content, $entity);
+	($content, $entity) = unmime(\@body) if $'Header{'Mime-Version'};
+
+	&'add_log("biffing $entity entity is $content")
+		if length($content) && $'loglvl > 8;
+
+	strip_html(\@body) if $content =~ /html\b/;
 	&trim(*body) if $trim;		# Smart trim of leading reply text
 	&mh(*body, $len) if $cf'biffmh =~ /^on/i;
 
+	my $reformat = $cf'biffnice =~ /^on/i;
+	my $width = $context[1];
+	my $tl = 8;					# tab length
+
 	while ($len > 0 && $lines > 0 && defined ($_ = shift(@body))) {
-		next if /^\W*$/ && $skipnl;
+		next if $skipnl && is_blank($_);
+		my $line_length = 0;
+		1 while s|\t|' ' x ($tl - length($`) % $tl)|e;	# Expand tabs
+		s/[\x0-\x1f\x7f]//g;					# Remove all control chars
+		if ($reformat) {
+			local @tmp;
+			&format($_, $width, *tmp);			# Format line into @tmp
+			@tmp = grep(!is_blank($_), @tmp) if $skipnl;
+			foreach my $l (@tmp) {
+				$line_length += length $l;		# Do not count newlines
+				$lines--;
+			}
+			$_ = join($n, @tmp);
+		} else {
+			$line_length = length $_;
+			$lines -= int($line_length / $width) + 1;
+			$lines++ if 0 == $line_length % $width;
+		}
 		# Check for overflow, in case we use mh-style biffing and no
 		# reformatting occurred: we may be facing a huge string!
 		if (length($_) > $len) {
@@ -250,8 +320,7 @@ sub body {
 		} else {
 			$res .= $_ . $n;
 		}
-		$len -= length($_);		# Nobody will quibble over missing newline...
-		$lines--;
+		$len -= $line_length;
 	}
 	$res .= "...more...$n" if @body > 0 || $len < 0;
 	chop($res);					# Remove final \n\r for macro substitution
@@ -385,32 +454,26 @@ sub mh {
 		$line .= $_ . ' ';
 	}
 	chop($line);				# Remove trailing extra space
-	$ary[0] = $line;			# Replace first body line with compacted string
+	$ary[0] = $line;			# This is all we keep
 
 	# We stopped compating at index $i - 1, and indices start at 0. This means
 	# lines in the range [0, $i-1] are now all stored as $ary[0], and lines
 	# from [1, $i-1] must be removed from the array ($i-1 lines).
+	# We keep the extra lines so that a "...more..." indication can be given
+	# if needed.
 
 	splice(@ary, 1, $i - 1);	# Remove lines that are now part of $ary[0]
-
-	# Now optionally reformat the first line so that it fits into 80 columns.
-	# The line is formatted into an array, and that array is spliced back
-	# into @ary.
-
-	return unless $cf'biffnice =~ /^on/i;
-	local(@tmp);
-	&format($line, *tmp);		# Format line into @tmp
-	splice(@ary, 0, 1, @tmp);	# Insert formatted string back
 }
 
-
-# Format body to fit into 78 columns by inserting the generated lines in an
+# Format body to fit into tty width by inserting the generated lines in an
 # array, one line per item.
 sub format {
-	local($body, *ary) = @_;	# Body to be formatted, array for result
+	# Body to be formatted, tty width, array for result
+	local($body, $width, *ary) = @_;
 	local($tmp);				# Buffer for temporary formatting
 	local($kept);				# Length of current line
-	local($len) = 79;			# Amount of characters kept
+	local($len) = $width - 1;	# Amount of characters kept
+	$len = 1 if $len < 1;		# Avoid infinite loop if bad parameter
 	# Format body, separating lines on [;,:.?!] or space.
 	while (length($body) > $len) {
 		$tmp = substr($body, 0, $len);		# Keep first $len chars
@@ -419,10 +482,202 @@ sub format {
 		$tmp =~ s/\s*$//;					# Remove trailing spaces
 		$tmp =~ s/^\s*//;					# Remove leading spaces
 		push(@ary, $tmp);					# Create a new line
-		$body = substr($body, $kept, 9999);
+		$body = substr($body, $kept, length $body);
 	}
 	push(@ary, $body);			# Remaining information on one line
 }
 
+# One-liner quoted-printable decoder
+# MUST be on one line to not be dataloaded (would mess $1 in the regexp)
+sub to_txt { my $l = shift; $l =~ s/=([\da-fA-F]{2})/pack('C', hex($1))/ge; $l }
+
+# Quick removal of quoted-printable escapes within the headers
+# We do not care about the charset and hope the tty will be able to display
+# the characters just fine.
+sub unquote_printable {
+	my ($l) = @_;
+	# The to_txt() routine being used MUST NOT be dataloaded or $1 would be
+	# reset to '' on the first invocation.  It's a perl bug (seen in 5.10)
+	$l =~ s/=\?[\w-]+?\?Q\?(.*?)\?=\s*/to_txt($1)/sieg && $l =~ s/_/ /g;
+	&'add_log("unquoted '$_[0]' to '$l'") if $'loglvl > 19 && $_[0] ne $l;
+	return $l;
+}
+
+# Un-MIME the body by removing all the MIME headers and looking for the
+# first text entity in the message.
+# The supplied array is updated in-place and will contain on return the
+# lines of the MIME entity that was retained.
+# Returns the type of the retained MIME entity and the number of the entity
+# for logging, saying "global" for the whole message.
+# NB: if no text part is found, the array will be empty upon return.
+sub unmime {
+	my ($aref) = @_;
+	my $content = $'Header{'Content-Type'};
+	$content =~ s/\(.*?\)\s*//g;		# Removed allowed RFC822 comments
+
+	&'add_log("global MIME content-type is $content") if $'loglvl > 16;
+	return ($content, "global") unless $content =~ m|^multipart/|i;
+
+	my ($boundary) = $content =~ /boundary=(\S+);/;
+	($boundary) = $content =~ /boundary=(\S+)/ unless length $boundary;
+	$boundary = $1 if $boundary =~ /^"(.*)"/ || $boundary =~ /^'(.*)'/;
+
+	# We do not perform a recursive MIME parsing here
+
+	my $entity_content;
+	my $header;
+
+	&'add_log("searching text part for biffing, boundary=$boundary")
+		if $'loglvl > 16;
+
+	my @entity;
+	my $grabbed = 0;
+	my $n = 0;
+
+	for (;;) {
+		unless ($grabbed) {
+			return undef unless skip_past($aref, $boundary);
+		}
+		$grabbed = 0;
+		$header = parse_header($aref);
+		$entity_content = lc($header->{'Content-Type'});
+		$entity_content =~ s/\(.*?\)\s*//g;
+		&'add_log("parsed entity header: content is $entity_content")
+			if $'loglvl > 19;
+		$n++;
+		if ($entity_content =~ m|^text/|) {
+			# We found (another) text part, collect it...
+			@entity = ();
+			my $end = !skip_past($aref, $boundary, \@entity);
+			$grabbed = 1;		# Avoid skipping at next loop iteration
+			last if $entity_content =~ m|^text/plain\b|;	# Found the best one
+			last if $end;
+		}
+	}
+
+	my $entity = "${n}th";
+	$entity =~ s/1th$/1st/;
+	$entity =~ s/2th$/2nd/;
+	$entity =~ s/3th$/3rd/;
+
+	&'add_log("kept $entity entity $entity_content for biffing")
+		if $'loglvl > 18;
+
+	# Maybe the entity bears a transfer encoding?
+	my $entity_encoding = $header->{'Content-Transfer-Encoding'};
+	$entity_encoding =~ s/\(.*?\)\s*//g;
+
+	# XXX code duplication with body_check(), factorize some day...
+	my $output;
+	my $error;
+
+	if ($entity_encoding =~ /^base64\s*$/i) {
+		base64'reset(length $'Header{'Body'});
+		foreach my $d (@entity) {
+			base64'decode($d);
+		}
+		$error = base64'error_msg();
+		$output = base64'output();
+	} elsif ($entity_encoding =~ /^quoted-printable\s*$/i) {
+		qp'reset(length $'Header{'Body'});
+		foreach my $d (@entity) {
+			qp'decode($d);
+		}
+		$error = qp'error_msg();
+		$output = qp'output();
+	} else {
+		$error = "no encoding";
+	}
+
+	my $error_msg = length($error) ? $error : "none";
+	&'add_log("decoded $entity entity ($entity_encoding), error=$error_msg")
+		if $'loglvl > 18;
+
+	if (length $error) {
+		@$aref = @entity;
+	} else {
+		@$aref = split(/\r?\n/, $$output);
+	}
+	return ($entity_content, $entity);
+}
+
+# Skip past named boundary in the supplied array
+# If $collect is a defined ARRAY ref, push there all the lines we see until
+# the next boundary.
+# Return false when we see the LAST boundary in the message, meaning there
+# are no more parts to consider.
+sub skip_past {
+	my ($aref, $boundary, $collect) = @_;
+	my $l;
+	while (defined ($l = shift @$aref)) {
+		return 0 if $l eq "--$boundary--";
+		return 1 if $l eq "--$boundary";
+		push(@$collect, $l) if defined $collect;
+	}
+	return undef;	# Not found
+}
+
+# Parse embedded MIME headers, returning hash ref
+sub parse_header {
+	my ($aref) = @_;
+	my %header;
+	my $val;
+	my $last_header;
+	my $l;
+	my $saw_something = 0;
+	while (defined ($l = shift @$aref)) {
+		last if $l =~ /^$/ && $saw_something;
+		$saw_something++;
+		if ($l =~ /^\s/) {
+			$l =~ s/^\s+/ /;
+			$header{$last_header} .= $l if length $last_header;
+		} elsif (my ($field, $value) = $l =~ /^([!-9;-~\w-]+):\s*(.*)/) {
+			$last_header = header'normalize($field);
+			if ($header{$last_header} ne '') {
+				$header{$last_header} .= "\n" . $value;
+			} else {
+				$header{$last_header} = $value;
+			}
+		}
+	}
+	return \%header;
+}
+
+# Strip HTML in-place and remove spurious blank lines
+# This is done only on a best-effort basis to make the biff output nice
+sub strip_html {
+	my ($aref) = @_;
+	my @out;
+	my $in_style = 0;
+	my $is_nl;
+	my $last_was_nl = 0;
+	my $l;
+
+	while (defined ($l = shift @$aref)) {
+		$in_style++ while $l =~ s/<style\b.*?>//;
+		$in_style-- while $l =~ s|</style>||;
+		next if $in_style;
+		$l =~ s/<[^\0]*?>//g;
+		$l =~ s/&(\w)cedil;/$1/g;	# Transform into ASCII...
+		$l =~ s/&(\w)acute;/$1/g;
+		$l =~ s/&(\w)grave;/$1/g;
+		$l =~ s/&(\w)circ;/$1/g;
+		$l =~ s/&(\w)uml;/$1/g;
+		$l =~ s/&quot;/'/g;
+		$l =~ s/&nbsp;/ /g;
+		$l =~ s/&#160;/ /g;       # Same as &nbsp;
+		# Corect only for the ASCII part...
+		$l =~ s/&#(\d+);/($1 > 31 && $1 < 256) ? chr($1) : "?"/ge;
+		$l =~ s/&amp;/&/g;        # Must come last
+		$l =~ s/^\s*//;
+		$is_nl = 0 == length($l);
+		next if $last_was_nl && $is_nl;
+		$last_was_nl = $is_nl;
+		push(@out, $l);
+	}
+
+	@$aref = @out;
+}
+
 package main;
 
