@@ -1,6 +1,6 @@
-;# $Id: history.pl,v 3.0.1.4 2001/03/13 13:14:32 ram Exp $
+;# $Id: history.pl 11 2008-05-28 15:10:12Z rmanfredi $
 ;#
-;#  Copyright (c) 1990-1993, Raphael Manfredi
+;#  Copyright (c) 1990-2006, Raphael Manfredi
 ;#  
 ;#  You may redistribute only under the terms of the Artistic License,
 ;#  as specified in the README file that comes with the distribution.
@@ -94,7 +94,7 @@ sub history_tag {
 		if ($time > 0) {			# Message already recorded
 			local($tagmsg) = $tag eq '' ? '' : " ($tag)";
 			&add_log("history duplicate <$msg_id>" . $tagmsg) if $loglvl > 6;
-			$seen++;
+			$seen++ unless history_ignore($msg_id, $tag);
 		} else {					# Record message (appending)
 			&dbr'update($host, 'HISTORY', 0, @regexp);
 		}
@@ -102,6 +102,27 @@ sub history_tag {
 	return $seen;					# Return seen status
 }
 
+# Look at whether we should ignore the duplicate if -U was given
+# We ignore the first match for a given tag, so if one of the tags here
+# was already recorded in %ignored_history_tag, we ignore the -U switch.
+# The reason is that different paths in the rules could lead to a UNIQUE
+# command that is meant to trap the fact the message was already seen...
+# Return TRUE if we need to ignore the duplicate for this time
+sub history_ignore {
+	my ($msg_id, $tag) = @_;
+	return 0 unless $disable_unique;		# return unless -U given
+	if ($ignored_history_tag{$tag}++) {
+		# We already ignored once for this tag
+		add_log("not ignoring this duplicate <$msg_id>$tagmsg despite -U")
+			if $loglvl > 6;
+		return 0
+	}
+	my $tagmsg = $tag eq '' ? '' : " ($tag)";
+	add_log("ignoring duplicate <$msg_id>$tagmsg since you gave -U")
+		if $loglvl > 6;
+	return 1;				# Ignore this duplicate
+}
+
 # Obsolete -- will be removed in next release
 sub history_record {
 	&history_tag();
